/*******************************************************************************
  Calibration

  'Calibrate' the wing absolute rotary encoders.
  First we unwrap the measured position by shifting the bits in order to avoid
  wrapping if this is needed.
  Then we determine the maximum position so we can make it match the maximum
  angle attainable.

 -------------------------------------------------------------------------------
  (c) Copyright 2022 University of Liege
  Author: Thomas Lambert <t.lambert@uliege.be>
  ULiege - Aeroelasticity and Experimental Aerodynamics
  MIT License
  Repo: https://gitlab.uliege.be/thlamb/mecharaptor-controller
*******************************************************************************/

void Calibrate() {
  String input;
  uint32_t calibTime = 6000;  // Calibration time in ms

  Serial.println("[INPUT]: Calibrate angles? (Y/n)");

  while (Serial.available() == 0) {}  // Wait for serial input
  input = Serial.readString();

  if (input.equalsIgnoreCase("n") or input.equalsIgnoreCase("no")) {
    Serial.println("[INFO]: No calibration required. Using default values.");

  } else {

    Serial.print("[WARNING]: Setup will run for ");
    Serial.print(calibTime / 1000);
    Serial.println("s to get the offset and maximum value.");
    Serial.println("[INPUT]: Type 'Y' to begin.");

    while (Serial.available() == 0) {}  // Wait for serial input

    if (input.equalsIgnoreCase("y")) {

      // Run ESC at min value
      gEsc1.write(40);
      gEsc2.write(40);

      GetAngleOffset(calibTime / 2);
      GetMaxWingPos(calibTime / 2);

      // Stop ESC
      gEsc1.write(0);
      gEsc2.write(0);

      Serial.println("[INFO]: Calibration done.");
      Serial.println("Maximum wing positions are: ");
      PrintArray(gMaxWingPos);

      Serial.println("Offsets are: ");
      PrintArray(gAngleOffset);
    } else {
      Serial.println("[INFO]: Calibration aborted. Using default values.");
    }
  }

  Serial.println("[INPUT]: Start acquisition then press 'y' to begin.");
  while (Serial.available() == 0) {}  // Wait for serial input


  // Flush Serial before continuing
  while (Serial.available()) {
    Serial.read();
  }
}

// Determine if wing position needs to be offset to prevent wrap-around
void GetAngleOffset(uint32_t calibTime) {

  unsigned int maxPos[4];
  unsigned int minPos[4];
  unsigned int newPos;

  for (uint32_t tStart = millis(); (millis() - tStart) < calibTime;) {

    // Find max and min wing position
    for (int i = 0; i < sizeof(gCSN_PINS); i++) {
      newPos = ReadSensor(gCSN_PINS[i], gENC_ORIENT[i], 0);
      if (i == 0) {
        maxPos[i] = newPos;
        minPos[i] = newPos;
      } else {
        if (newPos > maxPos[i]) {
          maxPos[i] = newPos;
        } else if (newPos < minPos[i]) {
          minPos[i] = newPos;
        }
      }
    }
  }

  // Determine if offset is needed
  for (int i = 0; i < sizeof(gCSN_PINS); i++) {
    if (maxPos[i] - minPos[i] > ENC_PRECISION / 2) {
      gAngleOffset[i] = 1;
    }
  }
}


unsigned int GetMaxWingPos(uint32_t calibTime) {

  unsigned int newPos;

  for (uint32_t tStart = millis(); (millis() - tStart) < calibTime;) {

    // Find max and min wing position
    for (int i = 0; i < sizeof(gCSN_PINS); i++) {
      newPos = ReadSensor(gCSN_PINS[i], gENC_ORIENT[i], gAngleOffset[i]);
      if (newPos > gMaxWingPos[i]) {
        gMaxWingPos[i] = newPos;
      }
    }
  }
}
